import { isScalarType, Kind } from "graphql";
import { getNodeName, requireGraphQLSchema } from "../../utils.js";
const RULE_ID = "no-scalar-result-type-on-mutation", rule = {
  meta: {
    type: "suggestion",
    hasSuggestions: !0,
    docs: {
      category: "Schema",
      description: "Avoid scalar result type on mutation type to make sure to return a valid state.",
      url: `https://the-guild.dev/graphql/eslint/rules/${RULE_ID}`,
      requiresSchema: !0,
      examples: [
        {
          title: "Incorrect",
          code: (
            /* GraphQL */
            `
            type Mutation {
              createUser: Boolean
            }
          `
          )
        },
        {
          title: "Correct",
          code: (
            /* GraphQL */
            `
            type Mutation {
              createUser: User!
            }
          `
          )
        }
      ]
    },
    schema: []
  },
  create(context) {
    const schema = requireGraphQLSchema(RULE_ID, context), mutationType = schema.getMutationType();
    return mutationType ? {
      [[
        `:matches(ObjectTypeDefinition, ObjectTypeExtension)[name.value=${mutationType.name}]`,
        "> FieldDefinition > .gqlType Name"
      ].join(" ")](node) {
        const typeName = node.value, graphQLType = schema.getType(typeName);
        if (isScalarType(graphQLType)) {
          let fieldDef = node.parent;
          for (; fieldDef.kind !== Kind.FIELD_DEFINITION; )
            fieldDef = fieldDef.parent;
          context.report({
            node,
            message: `Unexpected scalar result type \`${typeName}\` for ${getNodeName(fieldDef)}`,
            suggest: [
              {
                desc: `Remove \`${typeName}\``,
                fix: (fixer) => fixer.remove(node)
              }
            ]
          });
        }
      }
    } : {};
  }
};
export {
  rule
};
